/**
 *
 *
 * @type {MVC.Model}
 */
mtango.ui.DeviceCommandUI = mtango.ui.DeviceAttributeUI.extend("DeviceCommandUI",
    /* @Static */
    {
        kAvailableViews: {
            fld_cmd: 'engines/mTangoUI/views/fld_cmd.ejs'
        },
        attributes     : {
            src          : 'string',
            view_url     : 'string',
            initial_value: '*'
        },
        _validator:
            function(inst){
                return function() {
                    if (!inst.src) inst.errors.push("Required attribute[src] is missing!");
                }
            }
        ,
        /**
         *
         * @override {DeviceAttributeUI#_parse}
         * @private
         */
        _parse         : function($el) {
            console.debug('A command is found on the page. Parsing ...');
            var attributes = {
                $element: $el,
                errors: []
            };
            if($el.attr('id')) attributes.id = $el.attr('id');

            attributes.src = $el.attr('src');
            attributes.view_url = $el.attr('view') ? $el.attr('view') : this.kAvailableViews.fld_cmd;
            attributes.initial_value = $el.attr('initial_value');

            console.debug('src=' + attributes.src);
            console.debug('view=' + attributes.view_url);
            console.debug('initial_value=' + attributes.initial_value);

            return attributes;
        },
        /**
         *
         * @override
         * @private
         */
        _create_wrapped: function(proxy, name, errors) {
            if(!proxy.Class.commands[name]){
                errors.push('Device['+proxy.deviceProxy.device+'] has no command [' + name + ']');
                return null;
            }
            return proxy['create_new_DeviceCommand_' + name]();
        },
        /**
         * Creates this instance
         *
         * @param {mtango.DeviceCommand} deviceCommand
         * @param {string} view_url
         * @param {*} initial_value
         * @return {this}
         */
        create         : function(deviceCommand, view_url, initial_value) {
            if (arguments.length < 2) throw "At least two arguments are expected here: wrapped and view_url";

            var view = this._resolve_view_url_to_view(view_url);
            return this.create_as_existing({
                wrapped: deviceCommand,
                view_url     : view_url,
                view         : view,
                initial_value: initial_value
            });
        }
    },
    /* @Prototype */
    {
        view             : null,
        $element         : null,
        /**
         *
         * @param {MVC.View} v
         */
        set_view         : function(v) {
            this.view = v;
        },
        /**
         *
         * @param attributes
         * @constructor
         */
        init             : function(attributes) {
            if (!attributes.id) attributes.id = this.Class._ID++;
            this._super(attributes);
        },
        /**
         *
         * @param {jQuery} v
         */
        set_$element     : function(v) {
            this.$element = v;
        },
        /**
         * Opens argin popup
         */
        open             : function() {
            if (this.wrapped.inputType != 'DevVoid') {
                var dlgId = '#pre-rendered_' + this.element_id();
                $(dlgId).popup({
                    afterclose: MVC.Function.bind(function () {
                        var value = $('#dlg_write_arg_' + this.element_id(), this.$element).val();
                        if (!value) value = "N/A";
                        var $btn = $('#argin_' + this.element_id(), this.$element);
                            $btn.val('(' + value + ')');
                        $btn.parent().text('(' + value + ')').append($btn);
                        this.argin = value;
                    }, this)
                }).popup('open');
            }
        },
        /**
         * Executes this command with this.argin (or without)
         */
        execute          : function() {
            if (this.wrapped.inputType != 'DevVoid') {
                console.debug('Executing command['+ this.wrapped.toString()+'] with argin=' + this.argin);
                this.wrapped.execute(this.argin, this._callback());
            } else {
                console.debug('Executing command['+ this.wrapped.toString()+'] without argument');
                this.wrapped.execute(this._callback());
            }
        },
        /**
         *
         * @return {jQuery} an html representation of this command
         */
        create_element   : function() {
            if (this.$element) return this.$element;
            var html = ['<mtango:cmd id="', this.element_id(), '" src="', this.src, '" view="', this.view_url, "' initial_value='", this.initial_value, "'><\/mtango:cmd>"].join('');
            return this.$element = $(html);
        },
        /**
         * Shows this element
         */
        _show             : function() {
            this.wrapped.response = new mtango.Response({
                argout   : 'N/A',
                timestamp: Date.now()
            });
            this.argin = this.initial_value;
            this.update();
        },
        /**
         * @override
         */
        update:function(){
            if(!this.wrapped.response.valid()) {
                this.wrapped.response.argout = this.wrapped.response.errors[0];
                this.wrapped.errors = [];
            }
            this._super();
        }
    }
);

if (typeof DeviceCommandUI == 'undefined') DeviceCommandUI = mtango.ui.DeviceCommandUI;