/**
 *
 * @type {MVC.Model}
 */
mtango.ui.Page = MVC.Model.extend('Page',
    /* @Static */
    {
        views_url:"pages/" + MVC.app_name + "/",
        attributes:{},
        default_attributes:{}
    },
    /* @Prototype */
    {
        id : null,
        $page      :null,
        views:{
            header     :null,
            content    :null,
            footer     :null
        },
        $header     :null,
        $content    :null,
        $footer     :null,
        _ui        :null,
        set_$page  :function (v) {
            this.$page = v;
        },
        set_views :function (v) {
            this.views = v;
        },
        set_$header :function (v) {
            this.$header = v;
        },
        set_$content:function (v) {
            this.$content = v;
        },
        set_$footer :function (v) {
            this.$footer = v;
        },
        /**
         * @init_description creates a new page
         *
         * @param {String} page id
         * @constructor
         */
        init       :function (page) {
            var $page = $(new View({url:'engines/mTangoUI/views/empty_page.ejs'}).render({
                pageId:page
            })).appendTo(document.body);

            var attributes = {
                id     : page,
                $page  :$page,
                views  :{
                    header :new View({url:this.Class.views_url + page + '/header.ejs'}),
                    content:new View({url:this.Class.views_url + page + '/content.ejs'}),
                    footer :new View({url:this.Class.views_url + page + '/footer.ejs'})
                },
                $header:$("#" + page + " > .header"),
                $content:$("#" + page + " > .content"),
                $footer:$("#" + page + " > .footer"),
                _ui   : []
            };

            this._super(attributes);
        },
        /**
         * Loads this page in the document, i.e. renders header,content and footer (these are located in pages/[app]/[page.id]/)
         *
         * @param {Object} data
         */
        load       :function (data) {
            $('.header', this.$page).html(this.views.header.render(data));
            $('.content', this.$page).html(this.views.content.render(data));
            $('.footer', this.$page).html(this.views.footer.render(data));
            //TODO http://stackoverflow.com/a/10878355
            this.$page.page();//trigger('pagecreate');
            this.$page.on('pagehide',MVC.Function.bind(this._on_hide,this));
            this.$page.on('pageshow',MVC.Function.bind(this._on_show,this));
            this.publish('loaded', {instance:this});
            this.publish(this.id + '.loaded', {instance:this});
        },
        /**
         * Client may subscribe to 'Page.{pafe.id}.hide' event.
         *
         * Client code will be executed after pausing this page activities
         *
         * @private
         */
        _on_hide:function(){
            this._ui.forEach(function(ui){
                ui.pause();
            });
            this.publish(this.id + '.hide', {instance:this});
        },
        /**
         * Client may subscribe to 'Page.{pafe.id}.show' event.
         *
         * Client code will be executed before resuming this page activities
         *
         * @private
         */
        _on_show:function(){
            this.publish(this.id + '.show', {instance:this});
            this._ui.forEach(function(ui){
                ui.resume();
            });
        }
    }
);

if(typeof Page == 'undefined') Page = mtango.ui.Page;