/**
 *
 * @type {MVC.Model}
 */
mtango.ui.Plot = MVC.Model.extend('Plot',
    /* @Static */
    {
        _ID               : 1,
        kViewUrl          : 'engines/mTangoUI/views/fld_plot.ejs',
        kFldEmptyViewUrl  : 'engines/mTangoUI/views/fld_empty.ejs',
        attributes        : {
            label         : 'string',
            width         : 'int',
            height        : 'int',
            'refresh-rate': 'int',
            data          : 'mtango.ui.DeviceAttributeUIs',
            range         : 'int'
        },
        default_attributes: {
            id            : 0,
            label         : 'Plot',
            width         : 640,
            height        : 480,
            'refresh-rate': 500,
            range         : 100000
        },
        associations      : {
            has_many: ['mtango.ui.DeviceAttributeUI']
        },
        _parse            : function($el) {
            console.debug("A plot has been found on the page. Parsing ...");
            var attributes = {
                $element: $el,
                errors:[]
            };
            if($el.attr('id')) attributes.id = $el.attr('id');

            attributes.label = $el.attr('label');
            console.debug('label=' + attributes.label);
            attributes.range = parseInt($el.attr('range'));
            console.debug('range=' + attributes.range);
            attributes.width = parseInt($el.attr('width'));
            console.debug('width=' + attributes.width);
            attributes.height = parseInt($el.attr('height'));
            console.debug('height=' + attributes.height);
            attributes['refresh-rate'] = parseInt($el.attr('refresh-rate'));
            console.debug('refresh-rate=' + attributes['refresh-rate']);

            return attributes;
        },
        /**
         *
         * @param {jQuery} $el
         * @return {this}
         */
        from_html         : function($el) {
            if (typeof jQuery.plot == 'undefined') {
                return new this({
                    $element: $el,
                    errors  : ['jQuery flot library is not added. Plot wont work! Please add the library to your index.html']
                });
            }


            var attributes = this._parse($el);

            var instance = new this(attributes);

            instance.validate();
            if(!instance.valid()) return instance;

            instance.data = [];
            instance._plot_data_by_id = {};
            var fld_empty_view_url = this.kFldEmptyViewUrl;
            console.debug("Parsing plot's data elements...");
            $('mtango\\:plot-data', $el).each(function () {
                var $attr = $(this);
                var attr_ui;
                $attr.attr({
                    view: fld_empty_view_url
                });
                    attr_ui = mtango.ui.DeviceAttributeUI.from_html($attr);
                    //TODO check numerical data
                if(!attr_ui.valid()){
                    instance.errors.push('Can not create attribute[' + $attr.attr('src') + ']');
                    instance.errors.push(attr_ui.errors);
                    return;
                }

                instance.data.push(attr_ui);
                instance._plot_data_by_id[attr_ui.id] = [];
            });
            console.debug('Done parsing.');

            instance.view_url = this.kViewUrl;
            try {
                instance.view = new View({url: instance.view_url});
            } catch (e) {
                instance.errors.push(e.message);
            }

            $el.attr('id', instance.element_id());
            return instance;
        }
        //TODO create
    },
    /* @Prototype */
    {
        view                : null,
        /**
         * Options for plot
         *
         * Client may set these options on the instance before calling .show
         */
        plot_options        : null,
        $element            : null,
        $plot               : null,
        _is_active          : false,
        /**
         * @return {Array} []:{data,label}
         */
        _get_plot_data      : function () {
            var result = [];
            for (var i = 0, size = this.data.length; i < size; ++i) {
                result.push({
                    data : this._plot_data_by_id[this.data[i].id],
                    label: this.data[i].wrapped.label
                });
            }
            return result;
        },
        /**
         * An array of 2-elements arrays, i.e. [x0,y0],[x1,y1],..., referred by DeviceAttribute.id
         */
        _plot_data_by_id    : null,
        _maintenance_id     : 0,
        /**
         *
         * @param {MVC.View} v
         */
        set_view            : function(v) {
            this.view = v;
        },
        /**
         *
         * @param {jQuery} v
         */
        set_$element        : function(v) {
            this.$element = v;
        },
        /**
         * @param {jQuery.plot} v
         */
        set_$plot           : function(v) {
            this.$plot = v;
        },
        /**
         *
         * @param {Object} v -- id -> data array
         */
        set__plot_data_by_id: function(v) {
            this._plot_data_by_id = v;
        },
        /**
         *
         * @param {Object} attributes
         * @constructor
         */
        init                : function (attributes) {
            if (!attributes.id) attributes.id = this.Class._ID++;
            this._super(attributes);

            this.plot_options =  {
                legend: {
                    show: true
                },
                xaxis : {
                    mode       : "time",
                    timeformat : "%M:%S",
                    minTickSize: [this['refresh-rate'] / 1000, 'second']
                }
            };
        },
        /**
         * Initializes flot view of this plot
         *
         * @private
         */
        _initializeFlot     : function() {
            this.set_$plot($.plot($('#' + this.element_id() + '-placeholder', this.$element), this._get_plot_data(), this.plot_options));
        },
        /**
         * Starts to collect data from specified attributes according to specified method
         *
         * @private
         */
        _start_collect_data : function() {
            var me = this;
            var substitute_update = function () {
                me._plot_data_by_id[this.id].push(
                    [this.wrapped.response.timestamp, this.wrapped.response.argout]
                );
            };
            this.data.forEach(
                function (wrappedUI) {
                    wrappedUI.update = substitute_update;
                    wrappedUI.show();
                });
        },
        _start_plot_drawing : function() {
            setInterval(
                MVC.Function.bind(this.update, this),
                this['refresh-rate']
            );
        },
        _slice_data: function(){
            var now = Date.now();
            var to_be_deleted = [];
            for (var i = 0, i_size = this.data.length; i < i_size; ++i) {
                var plot_data_item = this._plot_data_by_id[this.data[i].id];
                var shifted_elements = [];
                //for each data item which is older than now - range shift it
                while (plot_data_item.length > 0 && plot_data_item[0][0] < (now - this.range))
                    shifted_elements.push(
                        plot_data_item.shift()
                    );

                if (shifted_elements.length > 0)
                    to_be_deleted.push({
                        data : shifted_elements,
                        label: this.data[i].wrapped.label
                    });
            }
            if (to_be_deleted.length > 0)
                this.publish('slice_data', {
                    plot: this,
                    data: to_be_deleted
                });
        },
        _start_maintenance  : function() {
            this._maintenance_id = setInterval(
                MVC.Function.bind(
                    this._slice_data, this),
                this['refresh-rate'] * 10);
        },
        _start_watch        : function() {
            var $watch = $('#plot-legend-' + this.element_id(), this.$element);

            function update_watch(d) {
                var hours = d.getHours();
                var minutes = d.getMinutes();

                $watch.text([hours, ':', minutes < 10 ? '0' : '',minutes].join(''));
            }

            var d = new Date();
            update_watch(d);

            var seconds = d.getSeconds();
            setTimeout(
                function () {
                    d = new Date();
                    update_watch(d);
                    setInterval(function () {
                        d = new Date();
                        update_watch(d);
                    }, 60001)
                },(60 - seconds) * 1000 + 1);
        },
        /**
         * Displays this plot and starts required routines
         */
        show                : function() {
            if (!this.valid()) {
                this.$element.html(new View({url: mtango.ui.ERROR_VIEW_URL}).render({
                    message: 'Failed to initialize mtango:plot',
                    data   : this
                }));
                return;
            }
            this.$element.html(this.view.render(this)).enhanceWithin();
            this._initializeFlot();
            this._start_collect_data();
            this._start_plot_drawing();
            this._start_maintenance();
            this._start_watch();
        },
        /**
         * Updates this plot's UI
         */
        update:function(){
            //save some CPU cycles to avoid unnecessary UI update
            if(!this._is_active) return;
            this.$plot.setData(this._get_plot_data());
            this.$plot.setupGrid();
            this.$plot.draw();
        },
        /**
         * Pauses this Plot activities - underlying data pauses its activity
         */
        pause:function(){
            this._is_active = false;
            this.data.forEach(function(data){
                data.pause();
            });

            clearInterval(this._maintenance_id);
        },
        /**
         * Resumes this Plot activities: underlying data resumes its activities, ui of this Plot is updated
         *
         */
        resume:function(){
            this._is_active = true;
            this.data.forEach(function(data){
                data.resume();
            });
            this.update();

            this._start_maintenance();
        }
    }
);

if (typeof Plot == 'undefined') Plot = mtango.ui.Plot;