/**
 *
 * @author Igor Khokhriakov <igor.khokhriakov@hzg.de>
 * @since 30.03.13
 */
/**
 * Cordova default lifecycle events:
 *
 * deviceready
 * pause
 * resume
 * backbutton
 * menubutton
 * searchbutton
 * startcallbutton
 * endcallbutton
 * volumedownbutton
 * volumeupbutton
 *
 * Events added by org.apache.cordova.battery-status:
 *
 * batterycritical
 * batterylow
 * batterystatus
 *
 * Events added by org.apache.cordova.network-information:
 *
 * online
 * offline
 *
 *
 * @type {MVC.Controller.Action}
 */
MVC.Controller.Action.DeviceEvent = MVC.Controller.Action.Event.extend(
    /* @static */
    {
        /**
         * Cordova default lifecycle events:
         *
         * deviceready
         * pause
         * resume
         * backbutton
         * menubutton
         * searchbutton
         * startcallbutton
         * endcallbutton
         * volumedownbutton
         * volumeupbutton
         *
         * Events added by org.apache.cordova.battery-status:
         *
         * batterycritical
         * batterylow
         * batterystatus
         *
         * Events added by org.apache.cordova.network-information:
         *
         * online
         * offline
         *
         *
         * matches "events$"
         */
        match: new 
RegExp("deviceready|pause|resume|online|offline|backbutton|batterycritical|batterylow|batterystatus|menubutton|searchbutton|startcallbutton|endcallbutton|volumedownbutton|volumeupbutton$")
    },
    /* @prototype */
    {
        /**
         *
         * @param {string} action_name
         * @param {Function} callback
         * @param {string} className
         */
        init: function(action_name, callback, className){
            this.action = action_name;
            this.callback = function(){
                try{
                    callback();
                }catch(e){
                    console.error("Callback has failed due to error -" + e);
                }
            };
            this.className = className;
            this.element = document;
            this.event_type = action_name;

            MVC.Event.observe(document, this.event_type, callback, this.fail);
        },
        fail:function(error){
            console.error(error);
            throw error;
        }
    }
);
